"""
GHOST — Generalizable Hyperspectral Observation & Segmentation Toolkit

Install:  pip install ghost-hsi
CLI:      ghost train_spt --data <file> --gt <file>
"""

__version__ = "0.1.5"
__author__  = "Ishu"
